@@ -54,10 +54,6 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-inet-connection.h>
 
-void errorf(GtkWidget *parent, const char *fmt, ...);
-void messagef(GtkWidget *parent, char *title, const char *fmt, ...);
-gboolean yes_no_f(GtkWidget *parent, char *title, const char *fmt, ...);
-
 #include "pos.h"
 #include "osm.h"
 #include "potlatch_conf.h"
@@ -100,6 +96,12 @@ typedef struct appdata_s {
   dbus_mm_pos_t mmpos;
 #endif
 
+  /* flags used to prevent re-appearence of dialogs */
+  struct {
+    gulong not;     /* bit is set if dialog is not to be displayed again */
+    gulong reply;   /* reply to be assumed if "not" bit is set */
+  } dialog_again;
+
   struct {
     char *import_path;
     GtkWidget *menu_item_import;
@@ -114,6 +116,9 @@ typedef struct appdata_s {
   GtkWidget *menu_item_wms_clear;
   GtkWidget *menu_item_wms_adjust;
 
+  GtkWidget *menu_item_map_hide_sel;
+  GtkWidget *menu_item_map_show_all;
+
 } appdata_t;
 
 #include "map.h"
@@ -131,5 +136,7 @@ typedef struct appdata_s {
 #include "gps.h"
 #include "wms.h"
 #include "josm_presets.h"
+#include "relation_edit.h"
+#include "misc.h"
 
 #endif // APPDATA_H
