@@ -177,8 +177,6 @@ void canvas_image_move(canvas_item_t *item, gint x, gint y,
 
 gint canvas_item_get_segment(canvas_item_t *item, gint x, gint y) {
 
-  printf("getting segment at %d %d\n", x, y);
-
   canvas_points_t *points = NULL;
   double line_width = 0;
   
@@ -189,11 +187,10 @@ gint canvas_item_get_segment(canvas_item_t *item, gint x, gint y) {
 
   if(!points) return -1;
 
-  printf("  points = %d\n", points->num_points);
-
   gint retval = -1, i;
   double mindist = 100;
   for(i=0;i<points->num_points-1;i++) {
+
 #define AX (points->coords[2*i+0])
 #define AY (points->coords[2*i+1])
 #define BX (points->coords[2*i+2])
@@ -201,13 +198,9 @@ gint canvas_item_get_segment(canvas_item_t *item, gint x, gint y) {
 #define CX ((double)x)
 #define CY ((double)y)
 
-    printf("  segment %d/%d->%d/%d\n", 
-	   (int)AX, (int)AY, (int)BX, (int)BY);
-
     double len2 = pow(BY-AY,2)+pow(BX-AX,2);
     double m = ((CX-AX)*(BX-AX)+(CY-AY)*(BY-AY)) / len2;
     
-    printf("  m = %f\n", m);
     /* this is a possible candidate */
     if((m >= 0.0) && (m <= 1.0)) {
 
@@ -217,9 +210,8 @@ gint canvas_item_get_segment(canvas_item_t *item, gint x, gint y) {
       else
 	n = fabs(sqrt(len2) * -(AX+m*(BX-AX)-CX)/(BY-AY)); 
 
-      printf("  n = %f (%f)\n", n, BX-AX);
-
-      /* check if this is actually on the line */
+      /* check if this is actually on the line and closer than anything */
+      /* we found so far */
       if((n <= line_width/2) && (n < mindist)) {
 	retval = i;
 	mindist = n;
@@ -227,9 +219,10 @@ gint canvas_item_get_segment(canvas_item_t *item, gint x, gint y) {
     }
  }
 
-  /* the last and first point are identical for polygons. goocanvas */
-  /* doesn't need that, but that's how OSM works and it saved us from */
-  /* having to check the last->first connection for polygons seperately */
+  /* the last and first point are identical for polygons in osm2go. */
+  /* goocanvas doesn't need that, but that's how OSM works and it saves */
+  /* us from having to check the last->first connection for polygons */
+  /* seperately */
 
   return retval;
 }
