@@ -83,14 +83,20 @@ static void diff_save_ways(way_t *way, xmlNodePtr root_node) {
 
   /* store all modfied ways */
   while(way) {
-    if(way->flags) { 
+    if(way->flags) {
       xmlNodePtr node_way = xmlNewChild(root_node, NULL, 
 					 BAD_CAST "way", NULL);
 
       diff_save_state_n_id(way->flags, node_way, way->id);
       
-      if(!(way->flags & OSM_FLAG_DELETED)) {
-	/* additional info is only required if the way hasn't been deleted */
+      if(way->flags & OSM_FLAG_HIDDEN) 
+	xmlNewProp(node_way, BAD_CAST "hidden", BAD_CAST "true");
+
+      /* additional info is only required if the way hasn't been deleted */
+      /* and of the dirty or new flags are set. (otherwise e.g. only */
+      /* the hidden flag may be set) */
+      if((!(way->flags & OSM_FLAG_DELETED)) && 
+	 (way->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW))) {
 	node_chain_t *node_chain = way->node_chain;
 	while(node_chain) {
 	  xmlNodePtr node_node = xmlNewChild(node_way, NULL, 
@@ -162,7 +168,7 @@ static void diff_save_relations(relation_t *relation, xmlNodePtr root_node) {
 
 
 /* return true if no diff needs to be saved */
-gboolean diff_is_clean(osm_t *osm) {
+gboolean diff_is_clean(osm_t *osm, gboolean honor_hidden_flags) {
   gboolean clean = TRUE;
 
   /* check if a diff is necessary */
@@ -174,7 +180,12 @@ gboolean diff_is_clean(osm_t *osm) {
 
   way_t *way = osm->way;
   while(way && clean) {
-    if(way->flags) clean = FALSE;
+    if(honor_hidden_flags) {
+      if(way->flags) clean = FALSE;
+    } else
+      if(way->flags & ~OSM_FLAG_HIDDEN) 
+	clean = FALSE;
+
     way = way->next;
   }
 
@@ -184,7 +195,7 @@ gboolean diff_is_clean(osm_t *osm) {
 void diff_save(project_t *project, osm_t *osm) {
   char *diff_name = g_strdup_printf("%s/%s.diff", project->path, project->name);
 
-  if(diff_is_clean(osm)) {
+  if(diff_is_clean(osm, TRUE)) {
     printf("data set is clean, removing diff if present\n");
     g_remove(diff_name);
     g_free(diff_name);
@@ -380,6 +391,17 @@ void diff_restore_way(xmlDoc *doc, xmlNodePtr node_node, osm_t *osm) {
 
   int state = xml_get_prop_state(node_node, "state");
 
+  /* handle hidden flag */
+  gboolean hidden = FALSE;
+  char *str = (char*)xmlGetProp(node_node, BAD_CAST "hidden");
+  if(str) { 
+    if(strcasecmp(str, "true") == 0) 
+      hidden = TRUE;
+
+    xmlFree(str);
+  }
+
+
   /* evaluate properties */
   way_t *way = NULL;
   switch(state) {
@@ -409,7 +431,7 @@ void diff_restore_way(xmlDoc *doc, xmlNodePtr node_node, osm_t *osm) {
 
   case OSM_FLAG_DIRTY:
     printf("  Valid id (DIRTY)\n");
-    
+
     if((way = osm_get_way_by_id(osm, id))) 
       way->flags |= OSM_FLAG_DIRTY;
     else
@@ -430,21 +452,32 @@ void diff_restore_way(xmlDoc *doc, xmlNodePtr node_node, osm_t *osm) {
   way->id = id;
     
   /* update node_chain */
+  if(hidden)
+    way->flags |= OSM_FLAG_HIDDEN;
     
-  /* way may be an existing way, so remove nodes to */
-  /* make space for new ones */
-  if(way->node_chain) {
-    printf("  removing existing nodes for diff nodes\n");
-    osm_node_chain_free(way->node_chain);
-    way->node_chain = NULL;
-  }
-  
+  gboolean installed_new_nodes = FALSE;
+
   /* scan for nodes */
   node_chain_t **node_chain = &way->node_chain;
   xmlNode *nd_node = NULL;
   for(nd_node = node_node->children; nd_node; nd_node = nd_node->next) {
     if(nd_node->type == XML_ELEMENT_NODE) {
       if(strcasecmp((char*)nd_node->name, "nd") == 0) {
+
+	/* only replace the original nodes if new nodes have actually been */
+	/* found. */
+	if(!installed_new_nodes) {
+	  /* way may be an existing way, so remove nodes to */
+	  /* make space for new ones */
+	  if(way->node_chain) {
+	    printf("  removing existing nodes for diff nodes\n");
+	    osm_node_chain_free(way->node_chain);
+	    way->node_chain = NULL;
+	  }
+
+	  installed_new_nodes = TRUE;
+	}
+
 	/* attach node to node_chain */
 	*node_chain = osm_parse_osm_way_nd(osm, doc, nd_node);
 	if(*node_chain) 
@@ -452,16 +485,25 @@ void diff_restore_way(xmlDoc *doc, xmlNodePtr node_node, osm_t *osm) {
       }
     }
   }
+
+  /* only replace tags if nodes have been found before. if no nodes */
+  /* were found this wasn't a dirty entry but e.g. only the hidden */
+  /* flag had been set */
+  if(installed_new_nodes) {
   
-  /* node may be an existing node, so remove tags to */
-  /* make space for new ones */
-  if(way->tag) {
-    printf("  removing existing tags for diff tags\n");
-    osm_tag_free(way->tag);
-    way->tag = NULL;
+    /* node may be an existing node, so remove tags to */
+    /* make space for new ones */
+    if(way->tag) {
+      printf("  removing existing tags for diff tags\n");
+      osm_tag_free(way->tag);
+      way->tag = NULL;
+    }
+    
+    way->tag = xml_scan_tags(doc, node_node->children, osm);
+  } else {
+    printf("  no nodes restored, way isn't dirty!\n");
+    way->flags &= ~OSM_FLAG_DIRTY;
   }
-  
-  way->tag = xml_scan_tags(doc, node_node->children, osm);
 }
 
 void diff_restore_relation(xmlDoc *doc, xmlNodePtr node_rel, osm_t *osm) {
@@ -623,6 +665,24 @@ void diff_restore(appdata_t *appdata, project_t *project, osm_t *osm) {
   }
  
   g_free(diff_name);
+
+
+  /* check for hidden ways and update menu accordingly */
+  gboolean something_is_hidden = FALSE;
+  way_t *way = osm->way;
+  while(!something_is_hidden && way) {
+    if(way->flags & OSM_FLAG_HIDDEN)
+      something_is_hidden = TRUE;
+
+    way = way->next;
+  }
+
+  if(something_is_hidden) {
+    printf("hidden flags have been restored, enable show_add menu\n");
+
+    statusbar_set(appdata, _("Some objects have been hidden"), TRUE);
+    gtk_widget_set_sensitive(appdata->menu_item_map_show_all, TRUE);
+  }
 }
   
 gboolean diff_present(project_t *project) {
