@@ -39,6 +39,28 @@ static void on_way_cut_clicked(GtkButton *button, gpointer data) {
   map_action_set((appdata_t*)data, MAP_ACTION_WAY_CUT);
 }
 
+static void on_relation_add_clicked(GtkButton *button, gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  relation_item_t relitem;
+
+  switch(appdata->map->selected.type) {
+  case MAP_TYPE_NODE:
+    relitem.type = NODE;
+    relitem.node = appdata->map->selected.node;
+    break;
+
+  case MAP_TYPE_WAY:
+    relitem.type = WAY;
+    relitem.way = appdata->map->selected.way;
+    break;
+    
+  default:
+    break;
+  }
+
+  relation_add_dialog(appdata, &relitem);
+}
+
 static void on_trash_clicked(GtkButton *button, gpointer data) {
   map_delete_selected((appdata_t*)data);
 }
@@ -68,6 +90,9 @@ void icon_bar_map_item_selected(appdata_t *appdata,
   gtk_widget_set_sensitive(iconbar->info, selected);
   gtk_widget_set_sensitive(iconbar->trash, selected);
 
+  if(iconbar->relation_add)
+    gtk_widget_set_sensitive(iconbar->relation_add, selected);
+
   if(selected && map_item && map_item->type == MAP_TYPE_WAY) {
     gtk_widget_set_sensitive(iconbar->way_node_add, TRUE);
     gtk_widget_set_sensitive(iconbar->way_cut, TRUE);
@@ -167,6 +192,13 @@ GtkWidget *iconbar_new(appdata_t *appdata) {
   iconbar->way_cut = tool_add(toolbar, appdata, "way_cut", 
 			      on_way_cut_clicked);
 
+#if 0
+  /* -------------------------------------------------------- */
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(),-1);
+
+  iconbar->relation_add = tool_add(toolbar, appdata, "relation_add", 
+			      on_relation_add_clicked);
+#endif
 
   gtk_box_pack_start(GTK_BOX(vbox), toolbar, TRUE, TRUE, 0);
 
