@@ -22,73 +22,6 @@
 
 #include "appdata.h"
 
-void messagef(GtkWidget *parent, char *title, const char *fmt, ...) {
-  va_list args;
-  va_start( args, fmt );
-  char *buf = g_strdup_vprintf(fmt, args);
-  va_end( args );
-
-  printf("%s: \"%s\"\n", title, buf); 
-  
-  GtkWidget *dialog = gtk_message_dialog_new(
-		     GTK_WINDOW(parent),
-                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                     GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                     buf);
-
-  gtk_window_set_title(GTK_WINDOW(dialog), title);
-
-  gtk_dialog_run(GTK_DIALOG(dialog));
-  gtk_widget_destroy(dialog);
-
-  g_free(buf);
-}
-
-gboolean yes_no_f(GtkWidget *parent, char *title, const char *fmt, ...) {
-  va_list args;
-  va_start( args, fmt );
-  char *buf = g_strdup_vprintf(fmt, args);
-  va_end( args );
-
-  printf("%s: \"%s\"\n", title, buf); 
-  
-  GtkWidget *dialog = gtk_message_dialog_new(
-		     GTK_WINDOW(parent),
-                     GTK_DIALOG_DESTROY_WITH_PARENT,
-		     GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-                     buf);
-
-  gtk_window_set_title(GTK_WINDOW(dialog), title);
-
-  gboolean yes = (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_YES);
-  gtk_widget_destroy(dialog);
-
-  g_free(buf);
-  return yes;
-}
-
-void errorf(GtkWidget *parent, const char *fmt, ...) {
-  va_list args;
-  va_start( args, fmt );
-  char *buf = g_strdup_vprintf(fmt, args);
-  va_end( args );
-
-  printf("errorf(\"%s\")\n", buf); 
-  
-  GtkWidget *dialog = gtk_message_dialog_new(
-		     GTK_WINDOW(parent),
-                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                     GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                     buf);
-
-  gtk_window_set_title(GTK_WINDOW(dialog), _("ERROR"));
-
-  gtk_dialog_run(GTK_DIALOG(dialog));
-  gtk_widget_destroy(dialog);
-
-  g_free(buf);
-}
-
 /******************** begin of menu *********************/
 
 static void 
@@ -128,6 +61,31 @@ cb_menu_upload(GtkWidget *window, gpointer data) {
   osm_upload(appdata, appdata->osm, appdata->project);
 }
 
+static void 
+cb_menu_download(GtkWidget *window, gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  if(!appdata->osm || !appdata->project) return;
+
+  /* redraw the entire map by destroying all map items and redrawing them */
+  track_save(appdata->project, appdata->track.track);
+  diff_save(appdata->project, appdata->osm);
+  map_clear(appdata);
+  osm_free(appdata->osm);
+
+  // download
+  osm_download(GTK_WIDGET(appdata->window), appdata->project);
+
+  appdata->osm = osm_parse(appdata->project->osm);
+  diff_restore(appdata, appdata->project, appdata->osm);
+  map_paint(appdata);
+
+  appdata->track.track = track_restore(appdata, appdata->project);
+  if(appdata->track.track)
+    map_track_draw(appdata->map, appdata->track.track);
+
+  wms_load(appdata);
+}
+
 static void 
 cb_menu_wms_import(GtkWidget *window, gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
@@ -146,6 +104,22 @@ cb_menu_wms_adjust(GtkWidget *window, gpointer data) {
   map_action_set(appdata, MAP_ACTION_BG_ADJUST);
 }
 
+/* ----------- hide objects for performance reasons ----------- */
+
+static void 
+cb_menu_map_hide_sel(GtkWidget *window, gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  map_hide_selected(appdata);
+}
+
+static void 
+cb_menu_map_show_all(GtkWidget *window, gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  map_show_all(appdata);
+}
+
+/* ----------------------------------------------- ----------- */
+
 #if 1  // mainly for testing
 static void 
 cb_menu_redraw(GtkWidget *window, gpointer data) {
@@ -174,6 +148,7 @@ cb_menu_save_changes(GtkWidget *window, gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
 
   diff_save(appdata->project, appdata->osm);
+  statusbar_set(appdata, _("Saved all changes in projects diff file"), FALSE);
 }
 
 
@@ -192,16 +167,16 @@ static void
 cb_menu_zoomin(GtkWidget *widget, appdata_t *appdata) {
   if(!appdata || !appdata->map) return;
 
-  map_set_zoom(appdata->map, appdata->map->zoom*ZOOM_FACTOR_MENU);
-  printf("zoom is now %f\n", appdata->map->zoom);
+  map_set_zoom(appdata->map, appdata->map->state->zoom*ZOOM_FACTOR_MENU, TRUE);
+  printf("zoom is now %f\n", appdata->map->state->zoom);
 }
 
 static void 
 cb_menu_zoomout(GtkWidget *widget, appdata_t *appdata) {
   if(!appdata || !appdata->map) return;
 
-  map_set_zoom(appdata->map, appdata->map->zoom/ZOOM_FACTOR_MENU);
-  printf("zoom is now %f\n", appdata->map->zoom);
+  map_set_zoom(appdata->map, appdata->map->state->zoom/ZOOM_FACTOR_MENU, TRUE);
+  printf("zoom is now %f\n", appdata->map->state->zoom);
 }
 #endif
 
@@ -315,17 +290,17 @@ void menu_create(appdata_t *appdata) {
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_upload), appdata);
 
-  item = gtk_menu_item_new_with_label( _("Save changes...") );
+  item = gtk_menu_item_new_with_label( _("Download...") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_download), appdata);
+
+  gtk_menu_append(GTK_MENU_SHELL(submenu), gtk_separator_menu_item_new());
+
+  item = gtk_menu_item_new_with_label( _("Save diff file") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_save_changes), 
 		   appdata);
 
-#if 1 // testing
-  item = gtk_menu_item_new_with_label( _("Redraw...") );
-  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
-  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_redraw), appdata);
-#endif
-
   /* -------------------- wms submenu -------------------- */
 
   item = gtk_menu_item_new_with_label( _("WMS") );
@@ -352,6 +327,32 @@ void menu_create(appdata_t *appdata) {
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_wms_adjust), 
 		   appdata);
 
+  /* -------------------- map submenu -------------------- */
+
+  item = gtk_menu_item_new_with_label( _("Map") );
+  gtk_menu_append(GTK_MENU_SHELL(menu), item);
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+  
+  appdata->menu_item_map_hide_sel = item = 
+    gtk_menu_item_new_with_label( _("Hide selected") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  gtk_widget_set_sensitive(item, FALSE);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_map_hide_sel), 
+		   appdata);
+
+  appdata->menu_item_map_show_all = item = 
+    gtk_menu_item_new_with_label( _("Show all") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  gtk_widget_set_sensitive(item, FALSE);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_map_show_all), 
+		   appdata);
+
+  gtk_menu_append(GTK_MENU_SHELL(submenu), gtk_separator_menu_item_new());
+
+  item = gtk_menu_item_new_with_label( _("Redraw...") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_redraw), appdata);
 
   /* -------------------- track submenu -------------------- */
 
@@ -436,8 +437,8 @@ void cleanup(appdata_t *appdata) {
 
   printf(" ok\n");
 
-  /* save a dirty project file */
-  if(appdata->project && appdata->project->dirty)
+  /* save project file */
+  if(appdata->project)
     project_save(GTK_WIDGET(appdata->window), appdata->project);
 
   map_remove_bg_image(appdata->map);
