@@ -21,6 +21,8 @@
 
 #include <gdk/gdkkeysyms.h>
 
+#undef DESTROY_WAIT_FOR_GTK
+
 static void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
   way->color = way->fill_color = MAP_COLOR_NONE;
   way->draw_flags = 0;
@@ -31,14 +33,12 @@ static void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
   if(!osm_way_is_way(way)) {
   
     /* check if this way is a polygon */
-    node_chain_t *node_chain = way->node_chain;
-    while(node_chain && node_chain->next) node_chain = node_chain->next;
-    if(way->node_chain && node_chain && 
-       way->node_chain->node == node_chain->node) {
-      way->draw_flags |= OSM_DRAW_FLAGS_POLYGON;
+    if(way->node_chain && 
+       (osm_way_get_last_node(way) == way->node_chain->node)) {
+      way->draw_flags |= OSM_DRAW_FLAG_POLYGON;
       
       /* default fill color is light grey */
-     way->fill_color = DEF_FILL_COLOR;
+      way->fill_color = DEF_FILL_COLOR;
     }
   }
   
@@ -49,7 +49,7 @@ static void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
 	way->color = (plcol->color << 8) | 0xff;
 
       if(plcol->stroke) 
-	way->draw_flags |= OSM_DRAW_FLAGS_STROKE;
+	way->draw_flags |= OSM_DRAW_FLAG_STROKE;
       
       if(plcol->fill_color != POTLATCH_COLOR_NONE)
 	way->fill_color = (plcol->fill_color << 8) | FILL_TRANSP;
@@ -97,6 +97,9 @@ static void map_statusbar(map_t *map, map_item_t *map_item) {
     break;
   }
 
+  gboolean collision = FALSE;
+  tag_t *tags = tag;
+
   if(id == ID_ILLEGAL) 
     str = g_strdup_printf(_("Unknown item"));
   else {
@@ -104,6 +107,9 @@ static void map_statusbar(map_t *map, map_item_t *map_item) {
 
     /* add some tags ... */
     while(tag) {
+      if(!collision && info_tag_key_collision(tags, tag))
+	collision = TRUE;
+
       /* we don't have much space, so ignore created_by tag */
       if(!osm_is_creator_tag(tag)) {
 	char *old = str;
@@ -114,7 +120,7 @@ static void map_statusbar(map_t *map, map_item_t *map_item) {
     }
   }
   
-  statusbar_set(map->appdata, str);
+  statusbar_set(map->appdata, str, collision);
   g_free(str);
 }
 
@@ -124,6 +130,12 @@ static void map_outside_error(appdata_t *appdata) {
 }
 
 static void map_item_chain_destroy(map_item_chain_t **chainP) {
+  if(!*chainP) {
+    printf("nothing to destroy!\n");
+    return;
+  }
+
+#ifdef DESTROY_WAIT_FOR_GTK
   map_item_chain_t *chain = *chainP;
   while(chain) {
     map_item_chain_t *next = chain->next;
@@ -131,7 +143,7 @@ static void map_item_chain_destroy(map_item_chain_t **chainP) {
     chain = next;
   }
 
-  /* way until gtks event handling has actually destroyed this item */
+  /* wait until gtks event handling has actually destroyed this item */
   printf("waiting for item destruction ");
   while(gtk_events_pending() || *chainP) {
     putchar('.');
@@ -140,8 +152,19 @@ static void map_item_chain_destroy(map_item_chain_t **chainP) {
   printf(" ok\n");
 
   /* the callback routine connected to this item should have been */
-  /* called by now and it should have set the chain to NULL */
-  //  g_assert(!*chainP);
+  /* called by now and it has set the chain to NULL */
+
+#else
+  map_item_chain_t *chain = *chainP;
+  while(chain) {
+    map_item_chain_t *next = chain->next;
+    canvas_item_destroy(chain->map_item->item);
+
+    g_free(chain);
+    chain = next;
+  }
+  *chainP = NULL;
+#endif
 }
 
 static void map_node_select(appdata_t *appdata, node_t *node) {
@@ -196,6 +219,7 @@ static void map_way_select(appdata_t *appdata, way_t *way) {
 
   map_statusbar(map, map_item);
   icon_bar_map_item_selected(appdata, map_item, TRUE);
+  gtk_widget_set_sensitive(appdata->menu_item_map_hide_sel, TRUE);
 
   node_chain_t *node_chain = map_item->way->node_chain;
   while(node_chain) {
@@ -241,7 +265,7 @@ static void map_way_select(appdata_t *appdata, way_t *way) {
     new_map_item->highlight = TRUE;
     
     map_hl_polyline_new(map, CANVAS_GROUP_WAYS_HL, new_map_item, points, 
-			(map_item->way->draw_flags & OSM_DRAW_FLAGS_STROKE)?
+			(map_item->way->draw_flags & OSM_DRAW_FLAG_STROKE)?
 	                 2*STROKE_WIDTH:3*WAY_WIDTH, HIGHLIGHT_COLOR);
 
     canvas_points_free(points);
@@ -265,11 +289,27 @@ static void map_item_select(appdata_t *appdata, map_item_t *map_item) {
 
 static void map_item_deselect(appdata_t *appdata) {
 
+  /* save tags for "last" function in info dialog */
+  if(appdata->map->selected.type == MAP_TYPE_NODE) {
+    if(appdata->map->last_node_tags) 
+      osm_tags_free(appdata->map->last_node_tags);
+
+    appdata->map->last_node_tags = 
+      osm_tags_copy(appdata->map->selected.node->tag, FALSE);
+  } else if(appdata->map->selected.type == MAP_TYPE_WAY) {
+    if(appdata->map->last_way_tags) 
+      osm_tags_free(appdata->map->last_way_tags);
+
+    appdata->map->last_way_tags = 
+      osm_tags_copy(appdata->map->selected.way->tag, FALSE);
+  }
+
   /* remove statusbar message */
-  statusbar_set(appdata, NULL);
+  statusbar_set(appdata, NULL, FALSE);
 
   /* disable/enable icons in icon bar */
   icon_bar_map_item_selected(appdata, NULL, FALSE);
+  gtk_widget_set_sensitive(appdata->menu_item_map_hide_sel, FALSE);
 
   /* remove highlight */
   map_hl_remove(appdata);
@@ -284,6 +324,7 @@ static gint map_item_destroy_event(GtkWidget *widget, gpointer data) {
 
   //  printf("destroying map_item @ %p\n", map_item);
 
+#ifdef DESTROY_WAIT_FOR_GTK
   /* remove item from nodes/ways map_item_chain */
   map_item_chain_t **chain = NULL;
   if(map_item->type == MAP_TYPE_NODE)
@@ -306,8 +347,9 @@ static gint map_item_destroy_event(GtkWidget *widget, gpointer data) {
   *chain = (*chain)->next;
 
   g_free(tmp);
-  g_free(map_item);
+#endif
 
+  g_free(map_item);
   return FALSE;
 }
 
@@ -367,7 +409,7 @@ static canvas_item_t *map_way_new(map_t *map, canvas_group_t group, way_t *way,
   map_item->type = MAP_TYPE_WAY;
   map_item->way = way;
 
-  if(way->draw_flags & OSM_DRAW_FLAGS_POLYGON) {
+  if(way->draw_flags & OSM_DRAW_FLAG_POLYGON) {
     map_item->item = canvas_polygon_new(map, group, points, 
 					width, color, fill_color);
   } else {
@@ -395,7 +437,7 @@ void map_show_node(map_t *map, node_t *node) {
 static void map_way_draw(map_t *map, way_t *way) {
 
   /* don't draw a way that's not there anymore */
-  if(way->flags & OSM_FLAG_DELETED)
+  if(way->flags & (OSM_FLAG_DELETED | OSM_FLAG_HIDDEN))
     return;
 
   /* allocate space for nodes */
@@ -415,14 +457,14 @@ static void map_way_draw(map_t *map, way_t *way) {
     }
     
     /* draw way */
-    if(way->draw_flags & OSM_DRAW_FLAGS_POLYGON) {
+    if(way->draw_flags & OSM_DRAW_FLAG_POLYGON) {
       map_way_new(map, CANVAS_GROUP_POLYGONS, way, points, 
 		  2, DARK_GREY, way->fill_color);
     } else {
       map_way_new(map, CANVAS_GROUP_WAYS, way, points, 
 		  WAY_WIDTH, way->color, NO_COLOR);
       
-      if(way->draw_flags & OSM_DRAW_FLAGS_STROKE)
+      if(way->draw_flags & OSM_DRAW_FLAG_STROKE)
 	map_way_new(map, CANVAS_GROUP_WAYS_OL, way, points, 
 		    STROKE_WIDTH, DARK_GREY, NO_COLOR);
     }
@@ -456,25 +498,21 @@ static void map_item_draw(map_t *map, map_item_t *map_item) {
 }
 
 static void map_item_remove(map_t *map, map_item_t *map_item) {
-  map_item_chain_t *chain = NULL;
+  map_item_chain_t **chainP = NULL;
 
   switch(map_item->type) {
   case MAP_TYPE_NODE:
-    chain = map_item->node->map_item_chain;
+    chainP = &map_item->node->map_item_chain;
     break;
   case MAP_TYPE_WAY:
-    chain = map_item->way->map_item_chain;
+    chainP = &map_item->way->map_item_chain;
     break;
   default:
     g_assert((map_item->type == MAP_TYPE_NODE) ||
 	     (map_item->type == MAP_TYPE_WAY));
   }
 
-  while(chain) {
-    map_item_chain_t *next = chain->next;
-    canvas_item_destroy(chain->map_item->item);
-    chain = next;
-  }
+  map_item_chain_destroy(chainP);
 }
 
 static void map_item_init(potlatch_conf_t *potlatch, map_item_t *map_item) {
@@ -519,13 +557,62 @@ static void map_draw(map_t *map, osm_t *osm) {
   }
 }
 
+void map_state_free(map_state_t *state) {
+  if(!state) return;
+  
+  /* free state of noone else references it */
+  if(state->refcount > 1) 
+    state->refcount--;
+  else
+    g_free(state);
+}
+
+void map_free_map_item_chains(appdata_t *appdata) {
+#ifndef DESTROY_WAIT_FOR_GTK
+  printf("  DESTROY_WAIT_FOR_GTK not set, removing all chains now\n");
+  
+  /* free all map_item_chains */
+  node_t *node = appdata->osm->node;
+  while(node) {
+    map_item_chain_t *chain = node->map_item_chain;
+    while(chain) {
+      map_item_chain_t *next = chain->next;
+      g_free(chain);
+      chain = next;
+    }
+    node->map_item_chain = NULL;
+    node = node->next;
+  }
+  
+  way_t *way = appdata->osm->way;
+  while(way) {
+    map_item_chain_t *chain = way->map_item_chain;
+    while(chain) {
+      map_item_chain_t *next = chain->next;
+      g_free(chain);
+      chain = next;
+    }
+    way->map_item_chain = NULL;
+    way = way->next;
+  }
+#endif
+}
+
 static gint map_destroy_event(GtkWidget *widget, gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
   map_t *map = appdata->map;
 
-  printf("destroying map\n");
+  printf("destroying entire map\n");
+  
+  map_free_map_item_chains(appdata);
 
+  if(map->last_node_tags) osm_tags_free(map->last_node_tags);
+  if(map->last_way_tags) osm_tags_free(map->last_way_tags);
+
+  map_state_free(map->state);
   g_free(map);
+
+
   appdata->map = NULL;
 
   return FALSE;
@@ -614,9 +701,18 @@ static map_item_t *map_real_item_at(map_t *map, gint x, gint y) {
   return map_item;
 }
 
-void map_set_zoom(map_t *map, double zoom) {
-  map->zoom = zoom;
-  canvas_set_zoom(map->canvas, map->zoom);
+void map_set_zoom(map_t *map, double zoom, 
+		  gboolean update_scroll_offsets) {
+  map->state->zoom = zoom;
+  canvas_set_zoom(map->canvas, map->state->zoom);
+
+  if(update_scroll_offsets) {
+    /* zooming affects the scroll offsets */
+    gint sx, sy;
+    canvas_get_scroll_offsets(map->canvas, &sx, &sy);
+    map->state->scroll_offset.x = sx;
+    map->state->scroll_offset.y = sy;
+  }
 }
 
 static gboolean map_scroll_event(GtkWidget *widget, GdkEventScroll *event,
@@ -625,9 +721,11 @@ static gboolean map_scroll_event(GtkWidget *widget, GdkEventScroll *event,
 
   if(event->type == GDK_SCROLL) {
     if(event->direction) 
-      map_set_zoom(appdata->map, appdata->map->zoom / ZOOM_FACTOR_WHEEL);
+      map_set_zoom(appdata->map, 
+		   appdata->map->state->zoom / ZOOM_FACTOR_WHEEL, TRUE);
     else
-      map_set_zoom(appdata->map, appdata->map->zoom * ZOOM_FACTOR_WHEEL);
+      map_set_zoom(appdata->map, 
+		   appdata->map->state->zoom * ZOOM_FACTOR_WHEEL, TRUE);
   }
 
   return TRUE;
@@ -659,6 +757,8 @@ static void map_do_scroll(map_t *map, gint x, gint y) {
   sx -= x-map->pen_down.at.x;
   sy -= y-map->pen_down.at.y;
   canvas_scroll_to(map->canvas, sx, sy);
+  map->state->scroll_offset.x = sx;
+  map->state->scroll_offset.y = sy;
 }
 
 /* scroll a certain step */
@@ -666,6 +766,8 @@ static void map_do_scroll_step(map_t *map, gint x, gint y) {
   gint sx, sy;
   canvas_get_scroll_offsets(map->canvas, &sx, &sy);
   canvas_scroll_to(map->canvas, sx+x, sy+y);
+  map->state->scroll_offset.x = sx+x;
+  map->state->scroll_offset.y = sy+y;
 }
 
 
@@ -807,7 +909,9 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
 
     printf("  dropped onto node #%ld\n", touchnode->id);
 
-    if(yes_no_f(GTK_WIDGET(appdata->window), _("Join nodes?"),
+    if(yes_no_f(GTK_WIDGET(appdata->window), 
+		appdata, MISC_AGAIN_ID_JOIN_NODES, 0,
+		_("Join nodes?"),
 		_("Do you want to join the dragged node with the one "
 		  "you dropped it on?"))) {
 
@@ -922,6 +1026,7 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
 	
       } else if(ways2join_cnt == 2) {
 	if(yes_no_f(GTK_WIDGET(appdata->window), 
+		    appdata, MISC_AGAIN_ID_JOIN_WAYS, 0,
 		    _("Join ways?"),
 		    _("Do you want to join the dragged way with the one "
 		      "you dropped it on?"))) {
@@ -1079,8 +1184,13 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
 	 node->lpos.x, node->lpos.y, node->pos.lat, node->pos.lon);
   
   /* now update the visual representation of the node */
-  canvas_item_t *item = map_item->item;
   
+  /* xyz */
+  map_item_chain_destroy(&node->map_item_chain);
+  map_node_draw(map, node);
+
+#if 0
+  canvas_item_t *item = map_item->item;
   if(!node->ways) 
     canvas_item_set_pos(item, &node->lpos, RADIUS);
   else {
@@ -1089,6 +1199,7 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
     if(!map_item->highlight)
       canvas_item_set_pos(item, &node->lpos, S_RADIUS);
   }
+#endif
 
   /* visually update ways, node is part of */
   way_t *way = osm->way;
@@ -1257,12 +1368,12 @@ static void map_bg_adjust(map_t *map, gint x, gint y) {
   g_assert(map->appdata->osm);
   g_assert(map->appdata->osm->bounds);
 
-  x += map->appdata->osm->bounds->min.x + map->bg_offset.x -
+  x += map->appdata->osm->bounds->min.x + map->bg.offset.x -
     map->pen_down.at.x;
-  y += map->appdata->osm->bounds->min.y + map->bg_offset.y -
+  y += map->appdata->osm->bounds->min.y + map->bg.offset.y -
     map->pen_down.at.y;
 
-  canvas_image_move(map->bg_item, x, y, map->bg_scale.x, map->bg_scale.y);
+  canvas_image_move(map->bg.item, x, y, map->bg.scale.x, map->bg.scale.y);
 }
 
 static void map_button_release(map_t *map, gint x, gint y) {
@@ -1272,8 +1383,8 @@ static void map_button_release(map_t *map, gint x, gint y) {
   switch(map->action.type) {
   case MAP_ACTION_BG_ADJUST:
     map_bg_adjust(map, x, y);
-    map->bg_offset.x += x - map->pen_down.at.x;
-    map->bg_offset.y += y - map->pen_down.at.y;
+    map->bg.offset.x += x - map->pen_down.at.x;
+    map->bg.offset.y += y - map->pen_down.at.y;
     break;
 
   case MAP_ACTION_IDLE:
@@ -1359,9 +1470,10 @@ static void map_button_release(map_t *map, gint x, gint y) {
     /* the last node placed is less than 5 pixels from the current */
     /* position */
     node_t *lnode = osm_way_get_last_node(map->action.way);
-    if(lnode && (map->zoom * sqrt((lnode->lpos.x-x)*(lnode->lpos.x-x)+ 
+    if(lnode && (map->state->zoom * sqrt((lnode->lpos.x-x)*(lnode->lpos.x-x)+ 
 				  (lnode->lpos.y-y)*(lnode->lpos.y-y))) < 5) {
       printf("detected double click -> simulate ok click\n");
+      map_hl_touchnode_clear(map);
       map_action_ok(map->appdata);
     } else {
       
@@ -1384,7 +1496,7 @@ static void map_button_release(map_t *map, gint x, gint y) {
 	switch(osm_node_chain_length(map->action.way->node_chain)) {
 	case 1:
 	  /* replace "place first node..." message */
-	  statusbar_set(map->appdata, _("Place next node of way"));
+	  statusbar_set(map->appdata, _("Place next node of way"), FALSE);
 	  break;
 	  
 	case 2:
@@ -1487,15 +1599,7 @@ static void map_button_release(map_t *map, gint x, gint y) {
 	}
 
 	/* ------------  copy all tags ------------- */
-	tag_t **dst_tag = &new->tag;
-	tag_t *src_tag = way->tag;
-	while(src_tag) {
-	  *dst_tag = g_new0(tag_t, 1);
-	  (*dst_tag)->key = g_strdup(src_tag->key);
-	  (*dst_tag)->value = g_strdup(src_tag->value);
-	  dst_tag = &(*dst_tag)->next;
-	  src_tag = src_tag->next;
-	}
+	new->tag = osm_tags_copy(way->tag, TRUE);
 
 	/* ---- transfer relation membership from way to new ----- */
 	relation_chain_t *rchain = osm_way_to_relation(map->appdata->osm, way);
@@ -1753,8 +1857,8 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
     case '+':
 #endif
       map_set_zoom(appdata->map, 
-		   appdata->map->zoom*ZOOM_FACTOR_BUTTON);
-      printf("zoom is now %f\n", appdata->map->zoom);
+		   appdata->map->state->zoom*ZOOM_FACTOR_BUTTON, TRUE);
+      printf("zoom is now %f\n", appdata->map->state->zoom);
       return TRUE;
       break;
 
@@ -1764,8 +1868,8 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
     case '-':
 #endif
       map_set_zoom(appdata->map, 
-		   appdata->map->zoom/ZOOM_FACTOR_BUTTON);
-      printf("zoom is now %f\n", appdata->map->zoom);
+		   appdata->map->state->zoom/ZOOM_FACTOR_BUTTON, TRUE);
+      printf("zoom is now %f\n", appdata->map->state->zoom);
       return TRUE;
       break;
 
@@ -1781,7 +1885,17 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 GtkWidget *map_new(appdata_t *appdata) {
   map_t *map = appdata->map = g_new0(map_t, 1);
 
-  map->zoom = 0.25;
+  if(appdata->project && appdata->project->map_state) {
+    printf("Using projects map state\n");
+    map->state = appdata->project->map_state;
+  } else {
+    printf("Creating new map state\n");
+    map->state = g_new0(map_state_t, 1);
+    map->state->zoom = 0.25;
+  }
+  
+  map->state->refcount++;
+
   map->pen_down.at.x = -1;
   map->pen_down.at.y = -1;
   map->appdata = appdata;
@@ -1841,7 +1955,7 @@ void map_init(appdata_t *appdata) {
   map_t *map = appdata->map;
 
   /* set initial zoom */
-  map_set_zoom(map, map->zoom);
+  map_set_zoom(map, map->state->zoom, FALSE);
   map_init_colors(appdata->potlatch, appdata->osm);
   map_draw(map, appdata->osm);
 
@@ -1849,7 +1963,11 @@ void map_init(appdata_t *appdata) {
 	appdata->osm->bounds->min.x, appdata->osm->bounds->min.y,
 	appdata->osm->bounds->max.x, appdata->osm->bounds->max.y);
 
-  canvas_scroll_to(map->canvas, 0,0);
+  printf("restore scroll offsets %d/%d\n",
+	 map->state->scroll_offset.x, map->state->scroll_offset.y);
+
+  canvas_scroll_to(map->canvas, 
+		   map->state->scroll_offset.x, map->state->scroll_offset.y);
 }
 
 
@@ -1877,6 +1995,8 @@ void map_clear(appdata_t *appdata) {
 
   printf("freeing map contents\n");
 
+  map_free_map_item_chains(appdata);
+
   /* remove a possibly existing highlight */
   map_item_deselect(appdata);
   
@@ -1958,7 +2078,7 @@ void map_action_set(appdata_t *appdata, map_action_t action) {
 
   g_assert(MAP_ACTION_NUM == sizeof(str_state)/sizeof(char*));
 
-  statusbar_set(appdata, str_state[action]);
+  statusbar_set(appdata, str_state[action], FALSE);
 }
 
 
@@ -1998,12 +2118,12 @@ void map_action_cancel(appdata_t *appdata) {
 
   case MAP_ACTION_BG_ADJUST:
     /* undo all changes to bg_offset */
-    map->bg_offset.x = appdata->project->wms_offset.x;
-    map->bg_offset.y = appdata->project->wms_offset.y;
+    map->bg.offset.x = appdata->project->wms_offset.x;
+    map->bg.offset.y = appdata->project->wms_offset.y;
 
-    gint x = appdata->osm->bounds->min.x + map->bg_offset.x;
-    gint y = appdata->osm->bounds->min.y + map->bg_offset.y;
-    canvas_image_move(map->bg_item, x, y, map->bg_scale.x, map->bg_scale.y);
+    gint x = appdata->osm->bounds->min.x + map->bg.offset.x;
+    gint y = appdata->osm->bounds->min.y + map->bg.offset.y;
+    canvas_image_move(map->bg.item, x, y, map->bg.scale.x, map->bg.scale.y);
     break;
 
   default:
@@ -2035,6 +2155,8 @@ void map_action_ok(appdata_t *appdata) {
       if(chain->node->id != ID_ILLEGAL) 
 	map_item_chain_destroy(&chain->node->map_item_chain);
 
+      map_node_draw(map, chain->node);
+
       /* we can be sure that no node gets inserted twice (even if twice in */
       /* the ways chain) because it gets assigned a non-ID_ILLEGAL id when */
       /* being moved to the osm node chain */
@@ -2058,8 +2180,8 @@ void map_action_ok(appdata_t *appdata) {
 
   case MAP_ACTION_BG_ADJUST:
     /* save changes to bg_offset in project */
-    appdata->project->wms_offset.x = map->bg_offset.x;
-    appdata->project->wms_offset.y = map->bg_offset.y;
+    appdata->project->wms_offset.x = map->bg.offset.x;
+    appdata->project->wms_offset.y = map->bg.offset.y;
     appdata->project->dirty = TRUE;
     break;
 
@@ -2074,6 +2196,12 @@ void map_action_ok(appdata_t *appdata) {
 void map_delete_selected(appdata_t *appdata) {
   map_t *map = appdata->map;
 
+  if(!yes_no_f(GTK_WIDGET(appdata->window), 
+	       appdata, MISC_AGAIN_ID_DELETE, MISC_AGAIN_FLAG_DONT_SAVE_NO,
+	       _("Delete selected object?"), 
+	       _("Do you really want to delete the selected object?")))
+    return;
+
   /* work on local copy since de-selecting destroys the selection */
   map_item_t item = map->selected;
 
@@ -2102,7 +2230,7 @@ void map_delete_selected(appdata_t *appdata) {
       }
 
       if(short_way) {
-	if(!yes_no_f(GTK_WIDGET(appdata->window), 
+	if(!yes_no_f(GTK_WIDGET(appdata->window), NULL, 0, 0,
 		     _("Delete node in short way(s)?"),
 		     _("Deleting this node will also delete one or more ways "
 		       "since they'll contain only one node afterwards. "
@@ -2260,9 +2388,9 @@ void map_track_pos(appdata_t *appdata, lpos_t *lpos) {
 void map_remove_bg_image(map_t *map) {
   if(!map) return;
 
-  if(map->bg_item) {
-    canvas_item_destroy(map->bg_item);
-    map->bg_item = NULL;
+  if(map->bg.item) {
+    canvas_item_destroy(map->bg.item);
+    map->bg.item = NULL;
   }
 }
 
@@ -2271,11 +2399,11 @@ static gint map_bg_item_destroy_event(GtkWidget *widget, gpointer data) {
 
   /* destroying background item */
 
-  map->bg_item = NULL;
-  if(map->bg_pix) {
+  map->bg.item = NULL;
+  if(map->bg.pix) {
     printf("destroying background item\n");
-    gdk_pixbuf_unref(map->bg_pix);
-    map->bg_pix = NULL;
+    gdk_pixbuf_unref(map->bg.pix);
+    map->bg.pix = NULL;
   }
   return FALSE;
 }
@@ -2285,17 +2413,56 @@ void map_set_bg_image(map_t *map, char *filename) {
 
   map_remove_bg_image(map);
 
-  map->bg_pix = gdk_pixbuf_new_from_file(filename, NULL);
+  map->bg.pix = gdk_pixbuf_new_from_file(filename, NULL);
 
   /* calculate required scale factor */
-  map->bg_scale.x = (float)(bounds->max.x - bounds->min.x)/
-    (float)gdk_pixbuf_get_width(map->bg_pix);
-  map->bg_scale.y = (float)(bounds->max.y - bounds->min.y)/
-    (float)gdk_pixbuf_get_height(map->bg_pix);
+  map->bg.scale.x = (float)(bounds->max.x - bounds->min.x)/
+    (float)gdk_pixbuf_get_width(map->bg.pix);
+  map->bg.scale.y = (float)(bounds->max.y - bounds->min.y)/
+    (float)gdk_pixbuf_get_height(map->bg.pix);
 
-  map->bg_item = canvas_image_new(map, CANVAS_GROUP_BG, map->bg_pix, 
-	  bounds->min.x, bounds->min.y, map->bg_scale.x, map->bg_scale.y);
+  map->bg.item = canvas_image_new(map, CANVAS_GROUP_BG, map->bg.pix, 
+	  bounds->min.x, bounds->min.y, map->bg.scale.x, map->bg.scale.y);
 
-  canvas_item_destroy_connect(map->bg_item, 
+  canvas_item_destroy_connect(map->bg.item, 
           G_CALLBACK(map_bg_item_destroy_event), map);
 }  
+
+
+/* -------- hide and show objects (for performance reasons) ------- */
+
+void map_hide_selected(appdata_t *appdata) {
+  map_t *map = appdata->map;
+  if(!map) return;
+
+  if(map->selected.type != MAP_TYPE_WAY) {
+    printf("selected item is not a way\n");
+    return;
+  }
+
+  way_t *way = map->selected.way;
+  printf("hiding way #%ld\n", way->id);
+
+  map_item_deselect(appdata);
+  way->flags |= OSM_FLAG_HIDDEN;
+  map_item_chain_destroy(&way->map_item_chain);
+
+  gtk_widget_set_sensitive(appdata->menu_item_map_show_all, TRUE);
+}
+
+void map_show_all(appdata_t *appdata) {
+  map_t *map = appdata->map;
+  if(!map) return;
+
+  way_t *way = appdata->osm->way;
+  while(way) {
+    if(way->flags & OSM_FLAG_HIDDEN) {
+      way->flags &= ~OSM_FLAG_HIDDEN;
+      map_way_draw(map, way);
+    }
+
+    way = way->next;
+  }
+
+  gtk_widget_set_sensitive(appdata->menu_item_map_show_all, FALSE);
+}
