@@ -96,11 +96,18 @@ typedef struct map_item_chain_s {
   struct map_item_chain_s *next;
 } map_item_chain_t;
 
+typedef struct {
+  gint refcount;
+  float zoom;                          // zoom level (1.0 = 1m/pixel
+  struct { gint x,y; } scroll_offset;  // initial scroll offset 
+} map_state_t;
+
 typedef struct map_s {
   appdata_t *appdata;
 
   canvas_t *canvas;
-  float zoom;                      // zoom level (1.0 = 1m/pixel
+
+  map_state_t *state;
 
   map_highlight_t *highlight;      // list of elements used for highlighting
 
@@ -111,10 +118,16 @@ typedef struct map_s {
   canvas_item_t *cursor;           // the cursor visible on the draw layer
   canvas_item_t *touchnode;        // the blue "touch node" on the draw layer
 
-  GdkPixbuf *bg_pix;
-  canvas_item_t *bg_item;
-  struct { float x, y; } bg_offset;
-  struct { float x, y; } bg_scale;
+  tag_t *last_node_tags;           // used to "repeat" tagging
+  tag_t *last_way_tags;
+  
+  /* background image related stuff */
+  struct {
+    GdkPixbuf *pix;
+    canvas_item_t *item;
+    struct { float x, y; } offset;
+    struct { float x, y; } scale;
+  } bg;
 
   struct {
     map_action_t type;            // current action type in progress
@@ -139,6 +152,7 @@ typedef struct map_s {
 } map_t;
 
 GtkWidget *map_new(appdata_t *appdata);
+void map_state_free(map_state_t *state);
 void map_init(appdata_t *appdata);
 gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event);
 void map_item_set_flags(map_item_t *map_item, int set, int clr);
@@ -168,6 +182,10 @@ void map_track_pos(appdata_t *appdata, lpos_t *lpos);
 /* background stuff */
 void map_set_bg_image(map_t *map, char *filename);
 void map_remove_bg_image(map_t *map);
-void map_set_zoom(map_t *map, double zoom);
+
+void map_hide_selected(appdata_t *appdata);
+void map_show_all(appdata_t *appdata);
+
+void map_set_zoom(map_t *map, double zoom, gboolean update_scroll_offsets);
 
 #endif // MAP_H
