@@ -534,7 +534,7 @@ static way_t *osm_parse_osm_way(osm_t *osm,
 
 /* ------------------- relation handling ------------------- */
 
-static void osm_member_free(member_t *member) {
+void osm_member_free(member_t *member) {
   if(member->role) g_free(member->role);
   g_free(member);
 }
@@ -1651,3 +1651,22 @@ void osm_way_rotate(way_t *way, gint offset) {
     chain->node->ways++; // increase way count of new start/end node
   }
 }
+
+tag_t *osm_tags_copy(tag_t *src_tag, gboolean update_creator) {
+  tag_t *new_tags = NULL;
+  tag_t **dst_tag = &new_tags;
+
+  while(src_tag) {
+    *dst_tag = g_new0(tag_t, 1);
+    (*dst_tag)->key = g_strdup(src_tag->key);
+    if(update_creator && (strcasecmp(src_tag->key, "created_by") == 0))
+      (*dst_tag)->value = g_strdup(PACKAGE " v" VERSION);
+    else
+      (*dst_tag)->value = g_strdup(src_tag->value);
+
+    dst_tag = &(*dst_tag)->next;
+    src_tag = src_tag->next;
+  }
+
+  return new_tags;
+}
