@@ -25,6 +25,7 @@
 #define OSM_FLAG_DIRTY    (1<<0)
 #define OSM_FLAG_DELETED  (1<<1)
 #define OSM_FLAG_NEW      (1<<2)
+#define OSM_FLAG_HIDDEN   (1<<3)
 
 typedef gulong item_id_t;
 
@@ -69,8 +70,8 @@ typedef struct node_chain {
   struct node_chain *next;
 } node_chain_t;
 
-#define OSM_DRAW_FLAGS_POLYGON  (1<<0)
-#define OSM_DRAW_FLAGS_STROKE   (1<<1)
+#define OSM_DRAW_FLAG_POLYGON  (1<<0)
+#define OSM_DRAW_FLAG_STROKE   (1<<1)
 
 typedef struct way_s {
   item_id_t id;
@@ -169,6 +170,7 @@ int osm_node_chain_length(node_chain_t *node_chain);
 void osm_node_free(node_t *node);
 
 void osm_members_free(member_t *member);
+void osm_member_free(member_t *member);
 
 void osm_tag_free(tag_t *tag);
 void osm_tags_free(tag_t *tag);
@@ -212,4 +214,6 @@ void osm_way_remove_from_relation(osm_t *osm, way_t *way);
 node_t *osm_way_get_last_node(way_t *way);
 void osm_way_rotate(way_t *way, gint offset);
 
+tag_t *osm_tags_copy(tag_t *tag, gboolean update_creator);
+
 #endif /* OSM_H */
