@@ -30,9 +30,10 @@
 #error "Tree not enabled in libxml"
 #endif
 
-#define WMS_FORMAT_PNG  (1<<0)
-#define WMS_FORMAT_GIF  (1<<1)
-#define WMS_FORMAT_JPG  (1<<2)
+#define WMS_FORMAT_JPG  (1<<0)
+#define WMS_FORMAT_JPEG (1<<1)
+#define WMS_FORMAT_PNG  (1<<2)
+#define WMS_FORMAT_GIF  (1<<3)
 
 typedef struct {
   pos_t min, max;
@@ -153,6 +154,8 @@ gboolean xmlTextIs(xmlDocPtr doc, xmlNodePtr list, char *str) {
   char *nstr = (char*)xmlNodeListGetString(doc, list, 1);
   if(!nstr) return FALSE;
 
+  printf("text = %s\n", nstr);
+
   gboolean match = (strcmp(str, nstr) == 0);  
   xmlFree(nstr);
   return match;
@@ -264,6 +267,8 @@ static wms_getmap_t *wms_cap_parse_getmap(xmlDocPtr doc, xmlNode *a_node) {
 	  wms_getmap->format |= WMS_FORMAT_GIF;
 	if(xmlTextIs(doc, cur_node->children, "image/jpg"))
 	  wms_getmap->format |= WMS_FORMAT_JPG;
+	if(xmlTextIs(doc, cur_node->children, "image/jpeg"))
+	  wms_getmap->format |= WMS_FORMAT_JPEG;
       } else 
 	printf("found unhandled "
 	       "WMT_MS_Capabilities/Capability/Request/GetMap/%s\n", 
@@ -274,8 +279,7 @@ static wms_getmap_t *wms_cap_parse_getmap(xmlDocPtr doc, xmlNode *a_node) {
   printf("Supported formats: %s%s%s\n", 
 	 (wms_getmap->format & WMS_FORMAT_PNG)?"png ":"",
 	 (wms_getmap->format & WMS_FORMAT_GIF)?"gif ":"",
-	 (wms_getmap->format & WMS_FORMAT_JPG)?"jpg ":"");
-
+	 (wms_getmap->format & (WMS_FORMAT_JPG | WMS_FORMAT_JPEG))?"jpg ":"");
   return wms_getmap;
 }
 
@@ -782,8 +786,8 @@ void wms_import(appdata_t *appdata) {
   }
 
   /* ... as well as in the map */
-  appdata->map->bg_offset.x = 0;
-  appdata->map->bg_offset.y = 0;
+  appdata->map->bg.offset.x = 0;
+  appdata->map->bg.offset.y = 0;
 
   /* get server from dialog */
   if(!wms_server_dialog(appdata, wms)) {
@@ -937,7 +941,8 @@ void wms_import(appdata_t *appdata) {
   while(!(wms->cap->request->getmap->format & (1<<format)))
     format++;
 
-  const char *formats[] = { "image/png", "image/gif", "image/jpg" };
+  const char *formats[] = { "image/jpg", "image/jpeg", 
+			    "image/png", "image/gif" };
 
   old = url;
   url = g_strdup_printf("%s&SRS=EPSG:4326&BBOX=%s,%s,%s,%s"
@@ -946,7 +951,7 @@ void wms_import(appdata_t *appdata) {
 			wms->height, formats[format]);
   g_free(old);
 
-  const char *exts[] = { "png", "gif", "jpg" };
+  const char *exts[] = { "jpg", "jpg", "png", "gif" };
   char *filename = g_strdup_printf("%s/wms.%s", appdata->project->path, 
 				   exts[format]);
 
@@ -984,16 +989,16 @@ void wms_load(appdata_t *appdata) {
 				     exts[i]);
 
     if(g_file_test(filename, G_FILE_TEST_EXISTS)) {
-      appdata->map->bg_offset.x = appdata->project->wms_offset.x;
-      appdata->map->bg_offset.y = appdata->project->wms_offset.y;
+      appdata->map->bg.offset.x = appdata->project->wms_offset.x;
+      appdata->map->bg.offset.y = appdata->project->wms_offset.y;
 
       map_set_bg_image(appdata->map, filename);
 
       /* restore image to saved position */
-      gint x = appdata->osm->bounds->min.x + appdata->map->bg_offset.x;
-      gint y = appdata->osm->bounds->min.y + appdata->map->bg_offset.y;
-      canvas_image_move(appdata->map->bg_item, x, y, 
-			appdata->map->bg_scale.x, appdata->map->bg_scale.y);
+      gint x = appdata->osm->bounds->min.x + appdata->map->bg.offset.x;
+      gint y = appdata->osm->bounds->min.y + appdata->map->bg.offset.y;
+      canvas_image_move(appdata->map->bg.item, x, y, 
+			appdata->map->bg.scale.x, appdata->map->bg.scale.y);
 
       g_free(filename);
 
